/////////////////////////////////////////////////////////////////////////////
// wincheck.cpp: application implementation file



#include "stdafx.h"
#include "resource.h"

#include "wincheck.h"

#include "dos.h"	
#include "direct.h"


/////////////////////////////////////////////////////////////////////////////
// COLOR MACROS

// Color RGB Definitions
#define BLACK			RGB(0x00,0x00,0x00)
#define WHITE			RGB(0xff,0xff,0xff)
#define DARKGREY		RGB(0x80,0x80,0x80)
#define GREY			RGB(0xc0,0xc0,0xc0)
#define BROWN			RGB(0x80,0x00,0x00)
#define RED				RGB(0xff,0x00,0x00)
#define YELLOW			RGB(0x80,0x80,0x00)
#define LIGHTYELLOW		RGB(0xff,0xff,0x00)
#define GREEN			RGB(0x00,0x80,0x00)
#define LIGHTGREEN		RGB(0x00,0xff,0x00)
#define CYAN			RGB(0x00,0x80,0x80)
#define LIGHTCYAN		RGB(0x00,0xff,0xff)
#define BLUE			RGB(0x00,0x00,0x80)
#define LIGHTBLUE		RGB(0x00,0x00,0xff)
#define MAGENTA			RGB(0x80,0x00,0x80)
#define FUCSIA			RGB(0xff,0x00,0xff)


COLORREF COLOR_CODES[] = {  BLACK,
							WHITE,
							DARKGREY,
							GREY,
							BROWN,
							RED,
							YELLOW,
							LIGHTYELLOW,
							GREEN,
							LIGHTGREEN,
							CYAN,
							LIGHTCYAN,
							BLUE,
							LIGHTBLUE,
							MAGENTA,
							FUCSIA };

static const char* COLOR_NAMES[] = {	"Black",
										"White",
										"DarkGrey",
										"Grey",
										"Brown",
										"Red",
										"Yellow",
										"LightYellow",
										"Green",
										"LightGreen",
										"Cyan",
										"LightCyan",
										"Blue",
										"LightBlue",
										"Magenta",
										"Fucsia" };

static const int NUM_COLORS = sizeof(COLOR_NAMES)/sizeof(COLOR_NAMES[0]);


// Color Global Functions
CString GetColorName(COLORREF color)
{
    for (int i=0; i<NUM_COLORS; i++)
    	if (color==COLOR_CODES[i])
    		return COLOR_NAMES[i];
    		
	return "Unknown Color";
}

COLORREF GetColorValue(const char* colorName)
{
	CString color(colorName);

	for (int i=0; i<NUM_COLORS; i++)
		if (color.CompareNoCase(COLOR_NAMES[i])==0)
			return COLOR_CODES[i];

	return BLACK;
}


/////////////////////////////////////////////////////////////////////////////
// Window Captions

static const char* CAPTIONS[] = {	"Free Memory",
									"Largest Block",
									"System Resources",
									"GDIs Resources",
									"User Resources",
									"CPU Type",
									"Math Coprocessor",
									"MS-DOS Version",
									"Windows Version",
									"Windows Mode",
									"Disk Space" };

static const int CAPTIONS_SIZE = sizeof(CAPTIONS) / sizeof(CAPTIONS[0]);
							
							
/////////////////////////////////////////////////////////////////////////////
// Create the window with the appropriate style, size, menu, etc.

CMainWindow::CMainWindow()
{                                                       
	// Init Data
	m_freeMemory = 0;
	m_freeMemoryBlock = 0;
	m_systemResources = 0;
	m_GDIResources = 0;
	m_userResources = 0;
	m_cpuType = "???";
	m_coprocessor = "???";
	m_MSDOSVersion = "??.?";
	m_winVersion = "??.?";
	m_runningMode = "???";
	m_diskSpace = 0;

	// Disable Title Bar
	CString className;       
	HBRUSH hBrush = (HBRUSH)(COLOR_WINDOW + 1);		// System Default
	HICON hIcon = AfxGetApp()->LoadIcon(AFX_IDI_STD_FRAME);
	HCURSOR hCursor = AfxGetApp()->LoadCursor(IDC_WINCHECKCURSOR);
	className = AfxRegisterWndClass(CS_DBLCLKS,hCursor,hBrush,hIcon);
		
    CString appName;
    appName.LoadString(AFX_IDS_APP_TITLE);

	if (!Create(className,appName,WS_BORDER|WS_POPUP))
	{
		AfxMessageBox("Can't Create Window.",MB_ICONSTOP);
		return;
	}

	// Create Private Font
	if (!m_font.CreateFont(-FONT_HEIGHT,0,		// Height & Width
							0,0,				// Escapement & Orientation
							FW_BOLD,			// Weight
							0,0,				// Italic, Underlined
							0,DEFAULT_CHARSET,	// StrickeOut, CharSet
							OUT_DEFAULT_PRECIS,
							CLIP_DEFAULT_PRECIS,
							DEFAULT_QUALITY,
							DEFAULT_PITCH,
							FONT_NAME))			// Face Name
	{
		AfxMessageBox("Can't Load Window Font.",MB_ICONSTOP);
		PostMessage(WM_CLOSE);
	}
}

void CMainWindow::InitMainWindow()
{
	// Calculate Window TabStop
	CClientDC dc(this);
	CFont* pOldFont = dc.SelectObject(&m_font);
	
	m_tabStop = 0;
	CSize textExt;
   	int lastInfo = (m_condensed) ? (LAST_CONDENSED_INFO + 1) : CAPTIONS_SIZE;
    
	for (int i=0; i<lastInfo; i++)	
	{
		textExt = dc.GetOutputTextExtent(CAPTIONS[i],_fstrlen(CAPTIONS[i]));
		m_tabStop = __max(m_tabStop,(textExt.cx + MARGIN_WIDTH));
	}

	CSize winSize;
	winSize.cx = 2*MARGIN_WIDTH + 2*m_tabStop;
	winSize.cy = 2*MARGIN_HEIGHT + lastInfo*(textExt.cy + 1);
	
	// Set Window Size & Pos
	const CWnd* pAfter = (m_alwaysOnTop) ? &wndTopMost : &wndBottom;
	SetWindowPos(pAfter,m_xPos,m_yPos,winSize.cx,winSize.cy,SWP_SHOWWINDOW);
	
	// Start the Timer
	if (!SetTimer(WINCHECK_TIMER_ID,m_updatingRate*1000,NULL))
	{
		AfxMessageBox("No Timers Available.",MB_ICONSTOP);
		PostMessage(WM_CLOSE);
	}

	// First Time Writing
	OnTimer(WINCHECK_TIMER_ID);

	// Select Default Font
	dc.SelectObject(pOldFont);
}

CMainWindow::~CMainWindow()
{
	m_font.DeleteObject();
	KillTimer(WINCHECK_TIMER_ID);
}


/////////////////////////////////////////////////////////////////////////////
// Mouse Management

void CMainWindow::OnLButtonDown(UINT nFlags, CPoint point)
{
	SetCapture();
	m_clicked = point;
	GetWindowRect(m_winPos);
	
	// Draw a Focus Rectangle
	HDC hDC = ::GetDC(NULL);
	::DrawFocusRect(hDC,&m_winPos);
	::ReleaseDC(NULL,hDC);

	CWnd::OnLButtonDown(nFlags, point);
}

void CMainWindow::OnMouseMove(UINT nFlags, CPoint point)
{
	if (GetCapture()==this)
	{
		// Delete Previous Focus Rectangle
		HDC hDC = ::GetDC(NULL);
		::DrawFocusRect(hDC,&m_winPos);
		
		CRect winRect;
		GetWindowRect(winRect);
		CPoint displ = point - m_clicked;
		m_winPos = winRect + displ;

		// Redraw the Focus Rectangle
		::DrawFocusRect(hDC,&m_winPos);
		::ReleaseDC(NULL,hDC);
	}
	else
		CWnd::OnMouseMove(nFlags, point);
}

void CMainWindow::OnLButtonUp(UINT nFlags, CPoint point)
{
	if (GetCapture()==this)
	{
		// Delete Previous Focus Rectangle
		HDC hDC = ::GetDC(NULL);
		::DrawFocusRect(hDC,&m_winPos);
		::ReleaseDC(NULL,hDC);

		CRect winRect;
		GetWindowRect(winRect);
		CPoint displ = point - m_clicked;
		m_winPos = winRect + displ;

		ReleaseCapture();
		MoveWindow(m_winPos);
		
		// Save Window New Position
		m_xPos = m_winPos.left;
		m_yPos = m_winPos.top;

		CTheApp* pApp = ((CTheApp*)AfxGetApp());
		pApp->WriteProfileInt(SECTION,X_POS_ENTRY,m_xPos);
		pApp->WriteProfileInt(SECTION,Y_POS_ENTRY,m_yPos);
	}	
	else
		CWnd::OnLButtonUp(nFlags, point);
}

void CMainWindow::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	CParameters dlg(this);
	dlg.m_ink = m_ink;
	dlg.m_paper = m_paper;
	dlg.m_condensed = m_condensed;
	dlg.m_alwaysOnTop = m_alwaysOnTop;
	dlg.m_updatingRate = m_updatingRate;
	
	if (dlg.DoModal()==IDOK)
	{
		m_ink = dlg.m_ink;
		m_paper = dlg.m_paper;
		m_condensed = dlg.m_condensed;
		m_alwaysOnTop = dlg.m_alwaysOnTop;
		m_updatingRate = dlg.m_updatingRate;
		
		KillTimer(WINCHECK_TIMER_ID);
		InitMainWindow();
	}
}


/////////////////////////////////////////////////////////////////////////////
// Data Updating

void CMainWindow::OnTimer(UINT nIDEvent)
{
	if ((GetCapture()!=this)
		&& (nIDEvent==WINCHECK_TIMER_ID))
	{
		// Memory
		m_freeMemory = ::GetFreeSpace(0) / 1024L;
		m_freeMemoryBlock = ::GlobalCompact((DWORD)0) / 1024L;

		// Windows Resources	
		m_systemResources = ::GetFreeSystemResources(GFSR_SYSTEMRESOURCES);
		m_GDIResources = ::GetFreeSystemResources(GFSR_GDIRESOURCES);
		m_userResources = ::GetFreeSystemResources(GFSR_USERRESOURCES);
	
		// Processors
		DWORD winFlags = ::GetWinFlags();
		m_cpuType = (winFlags & WF_CPU286) ? "80286" :
					(winFlags & WF_CPU386) ? "80386" :
					(winFlags & WF_CPU486) ? "i486" : "unknown";
		m_coprocessor = (winFlags & WF_80x87) ? "present" : "not present";

		// Software Versions
		char buffer[16];
		DWORD dwVersion = ::GetVersion();	
		m_MSDOSVersion = _itoa(HIBYTE(HIWORD(dwVersion)),buffer,10);
		m_MSDOSVersion += ".";
		m_MSDOSVersion += _itoa(LOBYTE(HIWORD(dwVersion)),buffer,10);
		m_winVersion = _itoa(LOBYTE(LOWORD(dwVersion)),buffer,10);
		m_winVersion += ".";
		m_winVersion += _itoa(HIBYTE(LOWORD(dwVersion)),buffer,10);
		
		// Running Mode
		m_runningMode = (winFlags & WF_ENHANCED) ? "enhanced" : "standard";
	
		// Disk
		struct _diskfree_t diskfree;
		if (_dos_getdiskfree(_getdrive(), &diskfree) == 0)
		{
			m_diskSpace = ((DWORD)diskfree.avail_clusters *
							(DWORD)diskfree.sectors_per_cluster *
							(DWORD)diskfree.bytes_per_sector / (DWORD)1024L);
		}
		else
			m_diskSpace = 0;
	
		// Rewrite Data
		Invalidate();
		UpdateWindow();
	}
	else	
		CWnd::OnTimer(nIDEvent);
}


/////////////////////////////////////////////////////////////////////////////
// Data Printing

void CMainWindow::OnPaint()
{
	CPaintDC dc(this); // device context for painting
	
	CRect winArea;
	GetClientRect(winArea);

	// Select the Background Brush
	CBrush bkBrush;
	bkBrush.CreateSolidBrush(m_paper);
	
	// Paint the Background
	dc.FillRect(winArea,&bkBrush);	
	bkBrush.DeleteObject();
	
	// Select Text Colors
	COLORREF oldBkColor = dc.SetBkColor(m_paper);
	COLORREF oldTextColor = dc.SetTextColor(m_ink);
	
	// Select Font
	CFont* pOldFont = dc.SelectObject(&m_font);
	
	// Write Info                      
	char buf[16];
	CString caption;
	int xCoor = MARGIN_WIDTH;
	int yCoor = MARGIN_HEIGHT;
	CSize sampleTextExt = dc.GetOutputTextExtent("ABC",3);
    int lastInfo = (m_condensed) ? (LAST_CONDENSED_INFO + 1) : CAPTIONS_SIZE;
    
	for (int i=0; i<lastInfo; i++)	
	{
		dc.TextOut(xCoor,yCoor,CAPTIONS[i],_fstrlen(CAPTIONS[i]));
		
		caption = " : ";
		switch (i)
		{
			case 0	:	caption += _ultoa(m_freeMemory,buf,10);
						caption += " KBytes";
						break;
			case 1	:	caption += _ultoa(m_freeMemoryBlock,buf,10);
						caption += " KBytes";
						break;
			case 2	:	caption += _itoa(m_systemResources,buf,10);
						caption += "% Free";
						break;
			case 3	:	caption += _itoa(m_GDIResources,buf,10);
						caption += "% Free";
						break;
			case 4	:	caption += _itoa(m_userResources,buf,10);
						caption += "% Free";
						break;
			case 5	:	caption += m_cpuType;
						break;
			case 6	:	caption += m_coprocessor;
						break;
			case 7	:	caption += m_MSDOSVersion;
						break;
			case 8	:	caption += m_winVersion;
						break;
			case 9	:	caption += m_runningMode;
						break;
			case 10	:	caption += _ultoa(m_diskSpace,buf,10);
						caption += " KBytes";
						break;
		}
        
		dc.TextOut(m_tabStop,yCoor,caption);
		yCoor += sampleTextExt.cy + 1;
	}
        
	// Restore DC Parameters        
	dc.SelectObject(pOldFont);
	dc.SetBkColor(oldBkColor);
	dc.SetTextColor(oldTextColor);
	
	ValidateRect(winArea);	
}


// CMainWindow message map:
BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
	//{{AFX_MSG_MAP( CMainWindow )
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()





/////////////////////////////////////////////////////////////////////////////
// CParameters dialog

CParameters::CParameters(CWnd* pParent /*=NULL*/)
	: CDialog(CParameters::IDD, pParent)
{
	//{{AFX_DATA_INIT(CParameters)
	m_updatingRate = 0;
	m_selInk = "";
	m_selPaper = "";
	//}}AFX_DATA_INIT
}

void CParameters::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CParameters)
	DDX_Control(pDX, IDC_PAPER, m_ctrlPaper);
	DDX_Control(pDX, IDC_INK, m_ctrlInk);
	DDX_Text(pDX, IDC_UPDATINGRATE, m_updatingRate);
	DDV_MinMaxInt(pDX, m_updatingRate, 1, 99);
	DDX_CBString(pDX, IDC_INK, m_selInk);
	DDX_CBString(pDX, IDC_PAPER, m_selPaper);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CParameters, CDialog)
	//{{AFX_MSG_MAP(CParameters)
	ON_BN_CLICKED(IDC_ABOUT, OnClickedAbout)
	ON_BN_CLICKED(IDC_CLOSE, OnClickedClose)
	ON_BN_CLICKED(IDC_SAVE, OnClickedSave)
	ON_BN_CLICKED(IDC_CONDENSED, OnClickedCondensed)
	ON_BN_CLICKED(IDC_ALWAYSONTOP, OnClickedAlwaysontop)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CParameters message handlers

BOOL CParameters::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	// Fill the Combo Boxes
	for (int i=0; i<NUM_COLORS; i++)
	{
		m_ctrlInk.AddString(COLOR_NAMES[i]);
		m_ctrlPaper.AddString(COLOR_NAMES[i]);
	}              
	
	m_ctrlInk.SelectString(-1,GetColorName(m_ink));
	m_ctrlPaper.SelectString(-1,GetColorName(m_paper));
	
	// Set Button States
	((CButton*)GetDlgItem(IDC_CONDENSED))->SetCheck(m_condensed);
	((CButton*)GetDlgItem(IDC_ALWAYSONTOP))->SetCheck(m_alwaysOnTop);
	
	return TRUE;
}

void CParameters::OnClickedAbout()
{
	CDialog dlg("ABOUTBOX",this);
	dlg.DoModal();
}

void CParameters::OnClickedClose()
{
	OnCancel();
	
	CWnd* pParent = GetParent();
	
	if (pParent!=NULL)
		pParent->PostMessage(WM_CLOSE);
		
}

void CParameters::OnClickedSave()
{
	UpdateData(TRUE);
	CTheApp* pApp = (CTheApp*)AfxGetApp();
	
	pApp->WriteProfileString(SECTION,INK_COLOR_ENTRY,m_selInk);
	pApp->WriteProfileString(SECTION,PAPER_COLOR_ENTRY,m_selPaper);
	pApp->WriteProfileInt(SECTION,CONDENSED_ENTRY,m_condensed);
	pApp->WriteProfileInt(SECTION,ALWAYS_ON_TOP_ENTRY,m_alwaysOnTop);
	pApp->WriteProfileInt(SECTION,UPDATING_RATE_ENTRY,m_updatingRate);
}

void CParameters::OnClickedCondensed()
{
	m_condensed = !m_condensed;
}

void CParameters::OnClickedAlwaysontop()
{
	m_alwaysOnTop = !m_alwaysOnTop;
}

void CParameters::OnOK()
{
	CDialog::OnOK();

	m_ink = GetColorValue(m_selInk);
	m_paper = GetColorValue(m_selPaper);
}




/////////////////////////////////////////////////////////////////////////////
// CTheApp

// InitInstance:
// When any CTheApp object is created, this member function is automatically
// called.  Any data may be set up at this point.
//
// Also, the main window of the application should be created and shown here.
// Return TRUE if the initialization is successful.
//
BOOL CTheApp::InitInstance()
{
	SetDialogBkColor();     // hook gray dialogs (was default in MFC V1)

	m_pMainWnd = new CMainWindow();
	m_pMainWnd->ShowWindow(m_nCmdShow);
	m_pMainWnd->UpdateWindow();

	// Read .INI Info
	CString str;
	CMainWindow* pWnd = (CMainWindow*)m_pMainWnd;
	pWnd->m_xPos = GetProfileInt(SECTION,X_POS_ENTRY,DEF_X_POS);
	pWnd->m_yPos = GetProfileInt(SECTION,Y_POS_ENTRY,DEF_Y_POS);
	pWnd->m_ink = GetColorValue(GetProfileString(SECTION,INK_COLOR_ENTRY,DEF_INK_COLOR));
	pWnd->m_paper = GetColorValue(GetProfileString(SECTION,PAPER_COLOR_ENTRY,DEF_PAPER_COLOR));
	pWnd->m_condensed = GetProfileInt(SECTION,CONDENSED_ENTRY,DEF_CONDENSED);
	pWnd->m_alwaysOnTop = GetProfileInt(SECTION,ALWAYS_ON_TOP_ENTRY,DEF_ALWAYS_ON_TOP);
	pWnd->m_updatingRate = GetProfileInt(SECTION,UPDATING_RATE_ENTRY,DEF_UPDATING_RATE);

	// Post-Creation Init
	pWnd->InitMainWindow();
	
	return TRUE;
}





/////////////////////////////////////////////////////////////////////////////

// theApp:
// Just creating this application object runs the whole application.
//
CTheApp NEAR theApp;





